import { formatsHandlersSelector } from '@/redux/configuration/configuration.selectors';
import { Button } from '@/shared/Button';
import { useSelect } from 'downshift';
import { useSelector } from 'react-redux';
import { SUBMAP_DOWNLOAD_HANDLERS_NAMES } from './DownloadSubmap.constants';
import { useGetSubmapDownloadUrl } from './utils/useGetSubmapDownloadUrl';

export const DownloadSubmap = (): JSX.Element => {
  const formatsHandlers = useSelector(formatsHandlersSelector);
  const formatsHandlersItems = Object.entries(formatsHandlers);
  const getSubmapDownloadUrl = useGetSubmapDownloadUrl();

  const { isOpen, getToggleButtonProps, getMenuProps } = useSelect({
    items: formatsHandlersItems,
  });

  return (
    <div className="relative">
      <Button
        data-testid="download-submap-button"
        variantStyles="ghost"
        className="mr-4"
        {...getToggleButtonProps()}
      >
        Download
      </Button>
      <ul
        data-testid="download-submap-list"
        className={`absolute left-[-50%] z-10 max-h-80 w-48 overflow-scroll rounded-sm border bg-white p-0 ps-0 ${
          !isOpen && 'hidden'
        }`}
        {...getMenuProps()}
      >
        {isOpen &&
          formatsHandlersItems.map(([formatId, handler]) => (
            <li key={formatId}>
              <a
                className="flex flex-col border-t px-4 py-2 shadow-sm"
                href={getSubmapDownloadUrl({ handler })}
                target="_blank"
                download
              >
                <span>{SUBMAP_DOWNLOAD_HANDLERS_NAMES[formatId]}</span>
              </a>
            </li>
          ))}
      </ul>
    </div>
  );
};
