import {
  ELEMENT_SEARCH_RESULT_MOCK_ALIAS,
  ELEMENT_SEARCH_RESULT_MOCK_REACTION,
} from '@/models/mocks/elementSearchResultMock';
import * as handleAliasResults from './handleAliasResults';
import * as handleReactionResults from './handleReactionResults';
import { handleSearchResultAction } from './handleSearchResultAction';

jest.mock('./handleAliasResults', () => ({
  __esModule: true,
  handleAliasResults: jest.fn().mockImplementation(() => (): null => null),
}));

jest.mock('./handleReactionResults', () => ({
  __esModule: true,
  handleReactionResults: jest.fn().mockImplementation(() => (): null => null),
}));

const handleAliasResultsSpy = jest.spyOn(handleAliasResults, 'handleAliasResults');
const handleReactionResultsSpy = jest.spyOn(handleReactionResults, 'handleReactionResults');

const POINT_MOCK = {
  x: 1323,
  y: 2000,
};
const ZOOM_MOCK = 3;
const MAX_ZOOM_MOCK = 9;

describe('handleSearchResultAction - util', () => {
  const dispatch = jest.fn();

  beforeEach(() => {
    jest.clearAllMocks();
  });

  describe('on ALIAS search results', () => {
    const searchResults = [ELEMENT_SEARCH_RESULT_MOCK_ALIAS];

    it('should fire handleAliasResults', async () => {
      await handleSearchResultAction({
        searchResults,
        dispatch,
        maxZoom: MAX_ZOOM_MOCK,
        isResultDrawerOpen: false,
        point: POINT_MOCK,
        zoom: ZOOM_MOCK,
      });
      expect(handleAliasResultsSpy).toBeCalled();
    });
  });

  describe('on REACTION search results', () => {
    const searchResults = [ELEMENT_SEARCH_RESULT_MOCK_REACTION];

    it('should fire handleReactionResults', async () => {
      await handleSearchResultAction({
        searchResults,
        dispatch,
        maxZoom: MAX_ZOOM_MOCK,
        isResultDrawerOpen: false,
        point: POINT_MOCK,
        zoom: ZOOM_MOCK,
      });
      expect(handleReactionResultsSpy).toBeCalled();
    });
  });
});
